% Generated by roxygen2 (4.1.1): do not edit by hand
% Please edit documentation in R/bioclim2.R
\name{bioclim2}
\alias{bioclim2}
\title{Bioclim - Bioclimatic Variables -- simplified}
\usage{
bioclim2(tmin = NULL, tmax = NULL, prec = NULL, tmean = NULL,
  cov = FALSE, files.as.inputs = TRUE)
}
\arguments{
\item{tmin}{a data.frame or matrix or filename of a csv or RData object
having 12 columns representing monthly or weekly minimum temperature data;
rows represent different locations.}

\item{tmax}{a data.frame or matrix or filename as with \code{tmin}
representing maximum temperature data.}

\item{prec}{a data.frame or matrix or filename as with \code{tmin}
representing precipitation data.}

\item{tmean}{a data.frame or matrix or filename as with \code{tmin}
representing mean temperature data.}

\item{cov}{a boolean value where TRUE represents Temperature seasonality (4)
is calculated as in ANUCLIM and FALSE is calclated as with Worldclim; see
details for further information.}

\item{files.as.inputs}{defines if the inputs are files.}
}
\value{
a matrix with columns representing variables requested and the
number of rows(and order of them) the same as the input \code{tmin},
\code{tmax}, \code{prec} or \code{tmean}.
}
\description{
\code{bioclim2} recreates the standard 19 bioclimatic variables (BIOCLIM),
as does related command \code{bioclim}, that were orginally generated using
ANUCLIM
\url{http://fennerschool.anu.edu.au/publications/software/anuclim.php} or
used by Worldclim \url{http://www.worldclim.org/}. \cr \cr These measures
are described below but include annual and quarterly summaries of
temperature and precipitation.\cr \cr \bold{NOTE:} the key differences
between this and \code{bioclim} is that: \cr 1. this is much faster due to C
implementation;\cr 2. this can only use monthly data;\cr 3. will only
generate all 19 variables; and\cr 4. can use file names rather than objects
in R to save on memory (not all things need to be loaded into memory at
once).\cr \cr !!!ASSUMES no missing data!!!
}
\details{
The variables created here are based primarily on BIOCLIM variables created
by ANUCLIM
\url{http://fennerschool.anu.edu.au/publications/software/anuclim.php}.
Below is a description of the variables and how they are calculated from the
ANUCLIM website, with Temperature Seasonality description modified for the
different method Worldclim uses.\cr\cr The descriptions below assume you are
using a weekly time step. If you are using months, the monthly values rather
than the weekly values will be used when calculating these parameters.\cr\cr
The quarterly parameters are not aligned to any calendar quarters. BIOCLIM's
definition of a quarter is any 13 consecutive weeks, (or any consecutive 3
months if running with a monthly time step). For example, the driest quarter
will be the 13 consecutive weeks that are drier than any other set of 13
consecutive weeks.\cr\cr \enumerate{ \item \bold{Annual Mean Temperature}\cr
The mean of all the weekly mean temperatures. Each weekly mean temperature
is the mean of that week's maximum and minimum temperature.  \item
\bold{Mean Diurnal Range(Mean(period max-min))}\cr The mean of all the
weekly diurnal temperature ranges. Each weekly diurnal range is the
difference between that week's maximum and minimum temperature.  \item
\bold{Isothermality 2/7}\cr The mean diurnal range (parameter 2) divided by
the Annual Temperature Range (parameter 7).  \item \bold{Temperature
Seasonality}\cr ANUCLIM (\code{cov}=TRUE) returns the temperature
Coefficient of Variation (C of V) as the standard deviation of the weekly
mean temperatures expressed as a percentage of the mean of those
temperatures (i.e. the annual mean). For this calculation, the mean in
degrees Kelvin is used. This avoids the possibility of having to divide by
zero, but does mean that the values are usually quite small.\cr Worldclim
(\code{cov}=FALSE) returns the the standard deviation of the weekly mean
temperatures.  \item \bold{Max Temperature of Warmest Period}\cr The highest
temperature of any weekly maximum temperature.  \item \bold{Min Temperature
of Coldest Period}\cr The lowest temperature of any weekly minimum
temperature.  \item \bold{Temperature Annual Range (5-6)}\cr The difference
between the Max Temperature of Warmest Period and the Min Temperature of
Coldest Period.  \item \bold{Mean Temperature of Wettest Quarter}\cr The
wettest quarter of the year is determined (to the nearest week), and the
mean temperature of this period is calculated.  \item \bold{Mean Temperature
of Driest Quarter}\cr The driest quarter of the year is determined (to the
nearest week), and the mean temperature of this period is calculated.  \item
\bold{Mean Temperature of Warmest Quarter}\cr The warmest quarter of the
year is determined (to the nearest week), and the mean temperature of this
period is calculated.  \item \bold{Mean Temperature of Coldest Quarter}\cr
The coldest quarter of the year is determined (to the nearest week), and the
mean temperature of this period is calculated.  \item \bold{Annual
Precipitation}\cr The sum of all the monthly precipitation estimates.  \item
\bold{Precipitation of Wettest Period}\cr The precipitation of the wettest
week or month, depending on the time step.  \item \bold{Precipitation of
Driest Period}\cr The precipitation of the driest week or month, depending
on the time step.  \item \bold{Precipitation Seasonality(C of V)}\cr The
Coefficient of Variation (C of V) is the standard deviation of the weekly
precipitation estimates expressed as a percentage of the mean of those
estimates (i.e. the annual mean).  \item \bold{Precipitation of Wettest
Quarter}\cr The wettest quarter of the year is determined (to the nearest
week), and the total precipitation over this period is calculated.  \item
\bold{Precipitation of Driest Quarter}\cr The driest quarter of the year is
determined (to the nearest week), and the total precipitation over this
period is calculated.  \item \bold{Precipitation of Warmest Quarter}\cr The
warmest quarter of the year is determined (to the nearest week), and the
total precipitation over this period is calculated.  \item
\bold{Precipitation of Coldest Quarter}\cr The coldest quarter of the year
is determined (to the nearest week), and the total precipitation over this
period is calculated.  }
}
\examples{
\dontrun{
#need to fill in
}
}
\author{
Jeremy VanDerWal \email{jjvanderwal@gmail.com}
}

